# Copyright (C) 2008-2009, Sebastian Riedel.

package Mojolicious::Script::Test;

use strict;
use warnings;

use base 'Mojo::Script::Test';

# Wow, there's a million aliens! I've never seen something so mind-blowing!
# Ooh, a reception table with muffins!

1;
__END__

=head1 NAME

Mojolicious::Script::Test - Test Script

=head1 SYNOPSIS

    use Mojo::Script::Test;

    my $test = Mojolicious::Script::Test->new;
    $test->run(@ARGV);

=head1 DESCRIPTION

L<Mojolicous::Script::Test> is a test script.

=head1 ATTRIBUTES

L<Mojolicious::Script::Test> inherits all attributes from
L<Mojo::Script::Test>.

=head1 METHODS

L<Mojolicious::Script::Test> inherits all methods from L<Mojo::Script::Test>.

=cut
