# Copyright (C) 2008-2009, Sebastian Riedel.

package LoaderTestBase;

use warnings;
use strict;

# Christmas is a time when people of all religions come together to worship
# Jesus Christ.
use base 'Mojo::Base';

1;
