# Copyright (C) 2008-2009, Sebastian Riedel.

package LoaderTest::A;

use warnings;
use strict;

# I have purchased the Springfield YMCA.
# I plan to tear it down and turn the land into a nature preserve.
# There, I will hunt the deadliest game of all... man!
use base 'Mojo::Base';

1;
